import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.ArrayList;
import java.util.List;
/**
 * Write a description of class Space here.
 * 
 * @author Anthony Hoang
 * @version (a version number or a date)
 */
public class Space  extends World
{
    public int roundPlace;
    public double px;
    public double py;
    public double px2;
    public double py2;
    private double width;
    private double height;
    public boolean showTail;
    public boolean paused;
    
    //---
    private int N = 2; //2 particles
    private int D = 2; // number of dimensions
    private double DT; // timestep
    private double pos[] = new double[D*N];
    private double vel[] = new double[D*N];
    private double accel[] = new double[D*N];
    private double mass[] = new double[N];
    private double T, T_final;
    private double rr_2[][] = new double[N][N];
    private int x = 0, y = 1, z =3;
    
    private Obj p1;
    private Obj p2;
    private ArrayList<Obj> objList = new ArrayList<Obj>();
    //----
    
    
    public Space()
    {    
        super(600, 400, 1); 
        showTail = true;
        paused = true;
        roundPlace = 1000;
        addObject(new Menu(), getWidth()-100, getHeight()/2);
        px = -2.5;
        py = -2.5;
        width = 5;
        height = 5;
        px2 = px+width;
        py2 = py + height;
        /*
        addObject(new Obj(100, 350), 0,0);
        addObject(new Obj(350, 100), 0,0);
        addObject(new Obj(100, 100), 0,0);
        addObject(new Obj(350, 350), 0,0);.
        */
        p1 = new Obj(0,0,0);
        p2 = new Obj(0,0,1);
        addObject(p1,0,0);
        addObject(p2,0,0);
        objList.add(p1);
        objList.add(p2);
        
        Coordinate c1 = new Coordinate(p1);
        addObject(c1,0,0);
        Coordinate c2 = new Coordinate(p2);
        addObject(c2,0,0);
        addObject(new XStart(), 100, getHeight()-20);
        addObject(new XEnd(),getWidth()-125, getHeight()-10);
        addObject(new YStart(), 125, getHeight()-5);
        addObject(new YEnd(), 100, 10);
        
        addObject(new GraphLine(0),0,0);
        addObject(new GraphLine(1),0,0);
        updateGraphCoord();
        addObject(new PauseWorld(), 559, 16);
        addObject(new TailsOnOff(), 502, 337);
        addObject(new NewObject(), 502, 280);
        addObject(new DTOptions(), 504, 225);
        addObject(new GraphWidthHeight(), 505, 115);
        addObject(new GraphView(), 503, 169);
        
        main();
        setPaintOrder(Menu.class);
    }
    public void act()
    {
        if (paused)
        return;
        
        for(int i = 0; i < D*N; i++)
        pos[i] += DT * vel[i];
        //Output(pos);
        T += DT;
        ComputeAccel(accel, pos);
        for(int i=0; i < D*N; i++)
        vel[i] += DT * accel[i];
        
        int posNum = 0;
        for( Obj o : objList)
        {
            o.setX(pos[posNum]);
            posNum++;
            o.setY(pos[posNum]);
            posNum++;
        }
        /*
        p1.setX(pos[0]);
        p1.setY(pos[1]);
        p2.setX(pos[2]);
        p2.setY(pos[3]);
        */
    }
    
    //--
    public double sqr(double x)
    {
        return x*x;
    }
    public double P3(double x)
    {
        return x*x*x;
    }
    public double X (double n[], int i)
    {
        return n[D*i];
    }
    public double Y (double n[],int i)
    {
        return n[D*i+1];
    }
    public double Z (double n[],int i)
    {
        return n[D*i+2];
    }
    public double r(double name[], int i, int alpha)
    {
        return name[D*i+alpha];
    }
    public double rs(double n[], int i, int j, int alpha)
    {
        return (r(n,j,alpha) - r(n,i,alpha));
    }
   public void ComputeAccel( double a[], double r[])
   {
       int i, j;
       //static double rr_2[N][N]; /* scalar distance^2 from body i to body j */

       for(i=0; i<N; i++)
       {
           rr_2[i][i] = 0.0; /* kludge alert */
           for(j=0; j<N; j++) if(i != j)
           {
               rr_2[i][j] = sqr(X(r,i)-X(r,j)) + sqr(Y(r,i)-Y(r,j));
               if(D>=3)
               rr_2[i][j] += sqr(Z(r,i)-Z(r,j));
            }
        }

        for(i=0; i<N; i++) /* compute accel of body i */
        {
            int alpha;
            for(alpha = x; alpha < x+D; alpha++)
            {
                int k;
                a[D*i+alpha] = 0.0;
                for(k=0; k<N; k++) if(k!=i)
                a[D*i+alpha] += mass[k]*rs(r,i,k,alpha)/P3(Math.sqrt(rr_2[i][k]));
            }
        }
    }

    public void Output(double r[])
    {
        System.out.println(T);
        for(int i = 0; i < N*D; i++)
        System.out.println(r[i]);
        System.out.println("\n");
    }
    public void main()
    {
        mass[0] = 0.99;
        mass[1] = 1-mass[0];
        pos[0] = -mass[1]; // x position of
        vel[0] = 0.0; // x velocity
        pos[1] = 0.0; // y position
        vel[1] = -mass[1]; // y velocity
        // Now, the small object
        pos[2] = mass[0]; // x pos
        vel[2] = 0.0; // x velocity
        pos[3] = 0.0; // y pos
        vel[3] = mass[0]; // y velocity
        
        // Now, the integration. First, advance the velocity by half a step.
        ComputeAccel(accel, pos);
        for(int i=0; i < D*N; i++)
        vel[i] += DT/2 * accel[i];

        
        int posNum = 0;
        for( Obj o : objList)
        {
            o.setX(pos[posNum]);
            posNum++;
            o.setY(pos[posNum]);
            posNum++;
        }
        DT = .001;
        T = 0; // time associated with positions (velocities are half a step ahead)
        T_final = 100;
    }
    public void addNewObject(double newX, double newY, double newXVel, double newYVel, double newMass)
    {
        
        N += 1;
        int NOLD = N-1;
        double posnew[] = new double[D*N];
        double velnew[] = new double[D*N];
        double accelnew[] = new double[D*N];
        double massnew[] = new double[N];
        double rr_2new[][] = new double[N][N];
        for (int i = 0; i < D*NOLD; i++)
        {
            posnew[i] = pos[i];
            velnew[i] = vel[i];
            accelnew[i] = accel[i];
        }
        for (int i = 0; i < NOLD; i++)
            massnew[i] = mass[i];
        for (int i = 0; i < NOLD; i++)
        {
            for (int j = 0; j < NOLD; j++)
            rr_2new[i][j] = rr_2[i][j];
        }
        pos = posnew;
        vel = velnew;
        accel = accelnew;
        mass = massnew;
        rr_2 = rr_2new;
        
        pos[D*N-2] = newX;
        pos[D*N-1] = newY;
        vel[D*N-2] = newXVel;
        vel[D*N-1] = newYVel;
        mass[N-1] = newMass;
        Obj newObj = new Obj(newX, newY,N-1);
        objList.add(newObj);
        addObject(newObj, (int)getXInPixel(newX), (int)getYInPixel(newY));
    }
    public void setTimeStep(double d)
    {
        DT = d;
    }
    
    public void setRoundPlace(int round)
    {
        roundPlace = round;
    }
    //--
    
    public boolean isInWorld (double dx, double dy)
    {
        boolean inX = px < dx && px2 > dx;
        boolean inY = py < dy && py2 > dy;
        return inX && inY;
        /*
        return Math.abs(x) < Math.abs(dx) 
        && Math.abs(x2) > Math.abs(dx)
        && Math.abs(y) < Math.abs(dy) 
        && Math.abs(y2) > Math.abs(dy);
        */
    }
    public boolean isInWorldX (double dx)
    {
        return px < dx && px2 > dx;
    }
    public boolean isInWorldY ( double dy)
    {
        return py < dy && py2 > dy;
    }
    public double getXInPixel(double dx)
    {
        double ax = Math.abs(px - dx);
        return (getWidth2()/width)*ax;
    }
    public double getYInPixel(double dy)
    {
        double ay = Math.abs(py2 - dy);
        return (getHeight()/height)*ay;
    }
    public int getWidth2()
    {
        return super.getWidth()-200;
    }
    public void setX(double ax)
    {
        px = ax;
        px2 = px+width;
        updateGraphCoord();
    }
    public void setY(double ay)
    {
        py = ay;
        py2 = py+height;
        updateGraphCoord();
    }
    public void setWidth(double w)
    {
        width = w;
        px2 = px+width;
        updateGraphCoord();
    }
    public void setHeight(double h)
    {
        height = h;
        py2 = py + height;
        updateGraphCoord();
    }
    public void updateGraphCoord()
    {
        List<GraphCoord> l = getObjects(GraphCoord.class);
        for (GraphCoord g : l)
        g.setCoord();
    }
    //----
    public void removeTails()
    {
        removeObjects(getObjects(Tracer.class));
    }
    public void openMenu()
    {
        
    }
}
